// CallLogRef.cpp: CCallLogRef NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CallLogRef.h"
#include "CallLogEntry.h"
#include "Debug.h"

/* Defines */
#define CALLEDHISTORY_PATH		_T("\\windows\\calledhistory.dat")
#define CALLINGHISTORY_PATH	_T("\\windows\\callinghistory.dat")


static
char*
ParseOneLine(
	char*	pBuf
)
{
	char*	p;
	char*	pNext;

	pNext = NULL;
	if (pBuf != NULL) {
		p = strchr(pBuf, '\r');
		if (p != NULL) {
			*p = '\0';
			p++;
			if (*p == '\n') {
				*p = '\0';
				p++;
			}

			pNext = p;
		}
	}

	return pNext;
}

static
char*
ParseOneItem(
	char*	pLine
)
{
	char*	p;
	char*	pNext;

	pNext = NULL;
	if (pLine != NULL) {
		p = strchr(pLine, '\t');
		if (p != NULL) {
			*p = '\0';
			pNext = p + 1;
		}
	}

	return pNext;
}

static
BOOL
LineToParsedItemArray(
	char*		pLine,
	char**		pAry,
	DWORD		dwArySize,
	LPDWORD		pdwOutSize
)
{
	BOOL	bRet = TRUE;
	DWORD	dwCount;
	LPSTR	pItem = NULL;
	LPSTR	pNextItem = NULL;

	dwCount = 0;
	pItem = pLine;
	while(pItem != NULL) {
		pNextItem = ParseOneItem(pItem);

		if (dwCount < dwArySize) {
			pAry[dwCount] = pItem;
			dwCount++;
		}
		else {
			bRet = FALSE;
			break;
		}

		pItem = pNextItem;
	}

	if (bRet) {
		*pdwOutSize = dwCount;
	}

	return bRet;
}


static
LPSTR
LoadFileToBuffer(
	LPTSTR	pFileName
)
{
	HANDLE	hFile			= INVALID_HANDLE_VALUE;
	LPSTR	pAnsiFileBuf	= NULL;
	LPSTR	pRet			= NULL;
	int		iBufSize		= 0;

	hFile = CreateFile(pFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {

		BY_HANDLE_FILE_INFORMATION	fileInfo;

		memset(&fileInfo, 0, sizeof(fileInfo));

		if (GetFileInformationByHandle(hFile, &fileInfo) && fileInfo.nFileSizeLow > 0) {
			try {
				pAnsiFileBuf = new char[fileInfo.nFileSizeLow];
				if (pAnsiFileBuf != NULL) {
					DWORD	dwReaded = 0;

					if (ReadFile(hFile, pAnsiFileBuf, fileInfo.nFileSizeLow, &dwReaded, NULL)) {
						pRet = pAnsiFileBuf;
					}
					else {
						delete [] pAnsiFileBuf;
						pAnsiFileBuf = NULL;
					}
				}
			}
			catch(...) {
				;
			}
		}

		CloseHandle(hFile);
		hFile = NULL;
	}

	return pRet;
}

static
void
DiscardFileBuffer(
	LPSTR	pBuf
)
{
	if (pBuf != NULL) {
		delete [] pBuf;
	}
}

static
LPWSTR
ConvertToWideStr(
	LPSTR	pStr
)
{
	int		iSize;
	LPWSTR	pRet = NULL;

	if (pStr == NULL) {
		return NULL;
	}

	iSize = MultiByteToWideChar(CP_ACP, 0, pStr, -1, NULL, 0);

	if (iSize > 0) {
		try {
			pRet = new WCHAR[(iSize + 1)*sizeof(WCHAR)];
			if (pRet != NULL)  {
				MultiByteToWideChar(CP_ACP, 0, pStr, -1, pRet, iSize + 1);
			}
		}
		catch(...) {
			;
		}
	}

	return pRet;
}


static
BOOL
LoadCalledHistoryList(
	StlCallLogEntryList&	calledLogList
)
{
	BOOL	bRet = FALSE;
	LPSTR	pBuf = NULL;

	DEBUGLOG((_T("LoadCalledHistoryList: In")));

	calledLogList.clear();

	pBuf = LoadFileToBuffer(CALLEDHISTORY_PATH);
	if (pBuf != NULL) {
		LPSTR	pLine = NULL;
		LPSTR	pNextLine = NULL;

		// 擪̃Rgs͓ǂݔ΂
		pLine = pBuf;
		pNextLine = ParseOneLine(pLine);
		if (pNextLine != NULL) {
			bRet = TRUE;

			pLine = pNextLine;

			while(pLine != NULL) {
				LPSTR	pItem = NULL;
				LPSTR	pNextItem = NULL;
				LPSTR	pItemAry[8];
				DWORD	dwCount = 0;

				pNextLine = ParseOneLine(pLine);

				memset(pItemAry, 0, sizeof(pItemAry));
				if (LineToParsedItemArray(pLine, pItemAry, sizeof(pItemAry)/sizeof(pItemAry[0]), &dwCount)) {
					if (dwCount == 8) {
						CCallLogEntry* pCallLogEntry = NULL;
						LPTSTR		pName	= NULL;
						LPTSTR		pNumber	= NULL;
						FILETIME	ftCallTime;
						SYSTEMTIME	systemTime;
						DWORD		dwFlag;
						int			nState;

						pNumber = ConvertToWideStr(pItemAry[0]);

						memset(&systemTime, 0, sizeof(systemTime));
						sscanf(pItemAry[2], "%04d%02d%02d%02d%02d", &systemTime.wYear, &systemTime.wMonth, &systemTime.wDay, &systemTime.wHour, &systemTime.wMinute);
						SystemTimeToFileTime(&systemTime, &ftCallTime);

						pName = ConvertToWideStr(pItemAry[4]);

						dwFlag = 0;
						nState = atoi(pItemAry[6]);
						if (nState == 1) {
							dwFlag |= CALLLOGENTRY_FLAG_MISSED;
						}
						else if (nState == 2) {
							dwFlag |= CALLLOGENTRY_FLAG_MISSED|CALLLOGENTRY_FLAG_RECORD;

						}

						pCallLogEntry = new CCallLogEntry(CALLLOGENTRY_TYPE_CALLED, pNumber, pName, &ftCallTime, dwFlag);

						if (pNumber != NULL) {
							delete [] pNumber;
							pNumber = NULL;
						}
						if (pName != NULL) {
							delete [] pName;
							pName = NULL;
						}

						if (pCallLogEntry != NULL) {
							calledLogList.push_back(pCallLogEntry);
						}

					}
				}

				pLine = pNextLine;
			}

		}

		DiscardFileBuffer(pBuf);
		pBuf = NULL;
	}

	DEBUGLOG((_T("LoadCalledHistoryList: count=%d"), calledLogList.size()));

	DEBUGLOG((_T("LoadCalledHistoryList: Out, rc=%d"), bRet));
	return bRet;
}

static
BOOL
LoadCallingHistoryList(
	StlCallLogEntryList&	callingLogList
)
{
	BOOL	bRet = FALSE;
	LPSTR	pBuf = NULL;

	DEBUGLOG((_T("LoadCallingHistoryList: In")));

	callingLogList.clear();

	pBuf = LoadFileToBuffer(CALLINGHISTORY_PATH);
	if (pBuf != NULL) {
		LPSTR	pLine = NULL;
		LPSTR	pNextLine = NULL;

		// 擪̃Rgs͓ǂݔ΂
		pLine = pBuf;
		pNextLine = ParseOneLine(pLine);
		if (pNextLine != NULL) {
			bRet = TRUE;

			pLine = pNextLine;

			while(pLine != NULL) {
				LPSTR	pItem = NULL;
				LPSTR	pNextItem = NULL;
				LPSTR	pItemAry[6];
				DWORD	dwCount = 0;

				pNextLine = ParseOneLine(pLine);

				memset(pItemAry, 0, sizeof(pItemAry));
				if (LineToParsedItemArray(pLine, pItemAry, sizeof(pItemAry)/sizeof(pItemAry[0]), &dwCount)) {
					if (dwCount == 6) {
						CCallLogEntry* pCallLogEntry = NULL;
						LPTSTR		pName	= NULL;
						LPTSTR		pNumber	= NULL;
						FILETIME	ftCallTime;
						DWORD		dwFlag;
						int			nState;

						pNumber = ConvertToWideStr(pItemAry[0]);

#if 0
						{
							FILETIME	ftLocalCallTime;
							SYSTEMTIME	localTime;

							memset(&localTime, 0, sizeof(localTime));
							sscanf(pItemAry[1], "%04d%02d%02d%02d%02d", &localTime.wYear, &localTime.wMonth, &localTime.wDay, &localTime.wHour, &localTime.wMinute);
							SystemTimeToFileTime(&localTime, &ftLocalCallTime);

							LocalFileTimeToFileTime(&ftLocalCallTime, &ftCallTime);
						}
#endif
//#if 0
						{
							SYSTEMTIME	systemTime;
							memset(&systemTime, 0, sizeof(systemTime));
							sscanf(pItemAry[1], "%04d%02d%02d%02d%02d", &systemTime.wYear, &systemTime.wMonth, &systemTime.wDay, &systemTime.wHour, &systemTime.wMinute);
							SystemTimeToFileTime(&systemTime, &ftCallTime);
						}
//#endif

						pName = ConvertToWideStr(pItemAry[3]);

						dwFlag = 0;
						nState = atoi(pItemAry[5]);
						if (nState >=3) {
							dwFlag |= CALLLOGENTRY_FLAG_BUNKEI;
							nState -=3;
						}
						if (nState == 1) {
							dwFlag |= CALLLOGENTRY_FLAG_184;
						}
						else if (nState == 2) {
							dwFlag |= CALLLOGENTRY_FLAG_186;
						}

						pCallLogEntry = new CCallLogEntry(CALLLOGENTRY_TYPE_CALLING, pNumber, pName, &ftCallTime, dwFlag);

						if (pNumber != NULL) {
							delete [] pNumber;
							pNumber = NULL;
						}
						if (pName != NULL) {
							delete [] pName;
							pName = NULL;
						}

						if (pCallLogEntry != NULL) {
							callingLogList.push_back(pCallLogEntry);
						}

					}
				}

				pLine = pNextLine;
			}

		}

		DiscardFileBuffer(pBuf);
		pBuf = NULL;
	}

	DEBUGLOG((_T("LoadCallingHistoryList: count=%d"), callingLogList.size()));

	DEBUGLOG((_T("LoadCallingHistoryList: Out, rc=%d"), bRet));
	return bRet;
}

static
void
DisposeLogList(
	StlCallLogEntryList&	logList
)
{
	while(!logList.empty()) {
		CCallLogEntry*	pCallLogEntry = logList.front();
		if (pCallLogEntry != NULL) {
			delete pCallLogEntry;
		}
		logList.pop_front();
	}
}

void
CCallLogRef::Dispose()
{
	while(!m_stlCallLogEntryList.empty()) {
		CCallLogEntry*	pCallLogEntry = m_stlCallLogEntryList.front();
		if (pCallLogEntry != NULL) {
			delete pCallLogEntry;
		}
		m_stlCallLogEntryList.pop_front();
	}
	m_CurrLogEntryItr = m_stlCallLogEntryList.begin();
}

CCallLogRef::CCallLogRef()
{
	m_bOpened = FALSE;
	m_stlCallLogEntryList.clear();
	m_CurrLogEntryItr = m_stlCallLogEntryList.begin();
}

CCallLogRef::~CCallLogRef()
{
	Dispose();
}

BOOL
CCallLogRef::Open()
{
	BOOL	bRet = FALSE;
	LPSTR	pBuf = NULL;
	StlCallLogEntryList	calledLogList;
	StlCallLogEntryList	callingLogList;

	DEBUGLOG((_T("CCallLogRef::Open: In")));

	if (!m_bOpened) {
		// M(calledhistory.dat)̉
		if (LoadCalledHistoryList(calledLogList)) {

			// M(callinghistory.dat)̉
			if (LoadCallingHistoryList(callingLogList)) {

				m_stlCallLogEntryList.clear();

				// 2̃Xg̃\[g&}[W
				calledLogList.reverse();
				callingLogList.reverse();
				while(!calledLogList.empty() || !callingLogList.empty()) {
					if (!calledLogList.empty() && !callingLogList.empty()) {
						CCallLogEntry*	pEntry1 = calledLogList.front();
						CCallLogEntry*	pEntry2 = callingLogList.front();

						if (CompareFileTime(pEntry1->GetTime(), pEntry2->GetTime()) >= 0) {
							m_stlCallLogEntryList.push_back(pEntry1);
							calledLogList.pop_front();
						}
						else {
							m_stlCallLogEntryList.push_back(pEntry2);
							callingLogList.pop_front();
						}
					}
					else {
						if (!calledLogList.empty()) {
							CCallLogEntry*	pEntry = calledLogList.front();
							m_stlCallLogEntryList.push_back(pEntry);
							calledLogList.pop_front();
						}
						else if (!callingLogList.empty()) {
							CCallLogEntry*	pEntry = callingLogList.front();
							m_stlCallLogEntryList.push_back(pEntry);
							callingLogList.pop_front();
						}
					}
				}

				m_CurrLogEntryItr = m_stlCallLogEntryList.begin();


				DEBUGLOG((_T("CCallLogRef::Open: succ count=%d"), m_stlCallLogEntryList.size()));

				m_bOpened = TRUE;
				bRet = TRUE;
			}
			else {
				DisposeLogList(calledLogList);
			}
		}
	}
	else {
		DEBUGLOG((_T("CCallLogRef::Open: err, already opened")));
	}

	DEBUGLOG((_T("CCallLogRef::Open: Out rc=%d"), bRet));

	return bRet;
}

void
CCallLogRef::Close()
{
	if (m_bOpened) {
		Dispose();
		m_bOpened = FALSE;
	}
}

BOOL
CCallLogRef::Seek(
	CALLLOG_REF_SEEK	seek,
	DWORD				dwRecord,
	LPDWORD				pdwRecord
)
{
	BOOL	bRet = FALSE;
	DWORD	dwCount = 0;

	if (pdwRecord == NULL) {
		return FALSE;
	}

	if (m_bOpened) {
		DWORD	dwPos;

		dwCount = m_stlCallLogEntryList.size();

		if (dwRecord < dwCount) {
			if (seek == CALLLOG_REF_BEGINNING) {
				dwPos = dwRecord;
				bRet = TRUE;
			}
			else if (seek == CALLLOG_REF_END) {
				dwPos = dwCount - dwRecord - 1;
				bRet = TRUE;
			}
		}

		if (bRet) {
			DWORD	i = 0;
			StlCallLogEntryListItr	itr;

			for (itr = m_stlCallLogEntryList.begin(); itr != m_stlCallLogEntryList.end(); itr++) {
				if (i == dwPos) {
					m_CurrLogEntryItr = itr;
					break;
				}

				i++;
			}

			*pdwRecord = dwPos;
		}
	}

	return bRet;
}

BOOL
CCallLogRef::Get(CCallLogEntry** ppCallLogEntry)
{
	BOOL	bRet;

	bRet = FALSE;

	if (ppCallLogEntry == NULL) {
		return FALSE;
	}

	if (m_bOpened) {
		if (m_CurrLogEntryItr != m_stlCallLogEntryList.end()) {
			*ppCallLogEntry = (*m_CurrLogEntryItr);
			m_CurrLogEntryItr++;
		}
		else {
			*ppCallLogEntry = NULL;
		}

		bRet = TRUE;
	}

	return bRet;
}